SamplerState sm:register(s1); // linear sampler
Texture2D input:register(t0); // output of blur x

cbuffer c : register(b1)
{
	float smooth; // 0.0 -> box filter, > 0.0 for gaussian
	float size; // length of the blur (global)
	float samplesx; // number of samples to take
	float directionx; // direction of blur
	float powerx; // length of the blur
	float samplesy; // number of samples to take
	float directiony; // direction of blur
	float powery; // length of the blur
	float blur_intensity; // how much of blur to mix
	float scene_intensity; // how much of scene to mix
}

float weight(float x) {
	return pow(2.71828, -(x*x)*(smooth*smooth)*64.0);
}

float hash(float2 t, float3 c) {
	float s = dot(frac(t.xyx*1132.914), frac(c.xyz*0.495));
	s = frac(s*45.723 + t.x*t.y*11.951312 + 11.8231);
	s = frac(s*11.423 + s*t.y*511.451312 + 1.5231);
	s = frac(s*11.423 + t.x*s*511.451312 + 1.5231);
	return s;
}

float4 main(float4 PositionSS : SV_Position, float2 t : TEXCOORD) : SV_TARGET0
{
	float samplesCount = samplesy * 255 + 1;
	float increment = 1.0 / samplesCount;
	float4 col = input.Sample(sm,t);

	float amount = size*size*powery;
	float pi = 355.0 / 113.0;
	float2 dir = float2(cos(directiony*pi),sin(directiony*pi));

	float wacc = weight(0);
	float4 acc = input.Sample(sm,t)*wacc;

	for (float x = increment; x<1.0; x += increment) {
		float w = weight(x);
		acc += input.Sample(sm,t + x*amount*dir)*w;
		wacc += w;
		w = weight(-x);
		acc += input.Sample(sm,t - x*amount*dir)*w;
		wacc += w;
	}

	float noise = hash(t,acc.xyz);
	float4 result = acc / wacc;
	result.xyz += noise/256.0;
	return result;
}